/*
 *  linux/fs/aclfs/dir.c
 *
 *  Copyright (C) 1991, 1992 Linus Torvalds
 *
 *  aclfs directory handling functions
 */

#include <asm/segment.h>

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/aclfs_fs.h>
#include <linux/stat.h>

static int aclfs_dir_read(struct inode * inode, struct file * filp, char * buf, int count)
{
	return -EISDIR;
}

static int aclfs_readdir(struct inode *, struct file *, struct dirent *, int);

static struct file_operations aclfs_dir_operations = {
	NULL,			/* lseek - default */
	aclfs_dir_read,		/* read */
	NULL,			/* write - bad */
	aclfs_readdir,		/* readdir */
	NULL,			/* select - default */
	NULL,			/* ioctl - default */
	NULL,			/* mmap */
	NULL,			/* no special open code */
	NULL			/* no special release code */
};

/*
 * directories can handle most operations...
 */
struct inode_operations aclfs_dir_inode_operations = {
	&aclfs_dir_operations,	/* default directory file-ops */
	aclfs_create,		/* create */
	aclfs_lookup,		/* lookup */
	aclfs_link,		/* link */
	aclfs_unlink,		/* unlink */
	aclfs_symlink,		/* symlink */
	aclfs_mkdir,		/* mkdir */
	aclfs_rmdir,		/* rmdir */
	aclfs_mknod,		/* mknod */
	aclfs_rename,		/* rename */
	NULL,			/* readlink */
	NULL,			/* follow_link */
	NULL,			/* bmap */
	aclfs_truncate,		/* truncate */
	NULL			/* permission */
};

static int aclfs_readdir(struct inode * inode, struct file * filp,
	struct dirent * dirent, int count)
{
	unsigned int offset,i;
	char c;
	struct buffer_head * bh;
	struct aclfs_dir_entry * de;

	if (!inode || !S_ISDIR(inode->i_mode))
		return -EBADF;
	if (filp->f_pos & (sizeof (struct aclfs_dir_entry) - 1))
		return -EBADF;
	while (filp->f_pos < inode->i_size) {
		offset = filp->f_pos & 1023;
		bh = aclfs_bread(inode,(filp->f_pos)>>BLOCK_SIZE_BITS,0);
		if (!bh) {
			filp->f_pos += 1024-offset;
			continue;
		}
		de = (struct aclfs_dir_entry *) (offset + bh->b_data);
		while (offset < 1024 && filp->f_pos < inode->i_size) {
			offset += sizeof (struct aclfs_dir_entry);
			filp->f_pos += sizeof (struct aclfs_dir_entry);
			if (de->inode) {
				for (i = 0; i < ACLFS_NAME_LEN; i++)
					if ((c = de->name[i]) != 0)
						put_fs_byte(c,i+dirent->d_name);
					else
						break;
				if (i) {
					put_fs_long(de->inode,&dirent->d_ino);
					put_fs_byte(0,i+dirent->d_name);
					put_fs_word(i,&dirent->d_reclen);
					brelse(bh);
					return i;
				}
			}
			de++;
		}
		brelse(bh);
	}
	return 0;
}
