/* read_acl.c */
/* (c) 1993 Gary Moyer -- All Rights Reserved */

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <asm/segment.h>
#include <linux/aclfs_fs.h>

int sys_readacl(char *filename, acl_s *acl[])
{
  struct inode *inode;
  int retval;


#ifdef VERBOSE_INODE_OPS
  printk("sys_readacl() -- reading acl\n");
#endif

  retval = namei(filename,&inode);

  if (retval)
    return retval;

  /* check to see if it's an ACLFS inode */
  if (!IS_ACLFS_INODE(inode)) 
  {
    iput(inode);
    return -EBADF;
  }

  if (!(current->euid == inode->i_uid) &&
      !(permission(inode,MAY_EXEC | MAY_READ)))
  {
    iput(inode);
    return -EACCES;
  }

  /* copy the contents over to user space */
  memcpy_tofs((void *) acl, 
              (void *) inode->u.aclfs_i.acl_list,
              (sizeof(acl_s) * MAX_ACLD));

  inode->i_dirt = 0;
  iput(inode);

  return 0;
}  
