/* read_acl.c */
/* (c) 1993 Gary Mopyer -- All Rights Reserved */

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <asm/segment.h>
#include <linux/aclfs_fs.h>

int sys_writeacl(const char *filename, acl_s *acl)
{
  struct inode *inode;
  int retval, acl_mode;


#ifdef VERBOSE_INODE_OPS
  printk("sys_writeacl() -- writing acl to file\n");
#endif

  retval = namei(filename,&inode);        /* get executable inode */

  if (retval)
    return retval; 

  /* check to see if it's an ACLFS inode */
  if (!IS_ACLFS_INODE(inode)) 
  {
    iput(inode);
    return -EBADF;
  }

  /* only owner or root can write the ACL */
  if (!(current->euid == inode->i_uid) && !(suser()))
  {
    iput(inode);
    return -EACCES;
  }


  /* copy the contents over from user space */
  memcpy_fromfs((void *) inode->u.aclfs_i.acl_list,
		         (void *) acl, 
                         (sizeof(acl_s) * MAX_ACLD));
  
  /* ..and take care of the object entries */
  /* looking for a difference here is just a waste of CPU */
  acl_mode = 0;

  acl_mode |= ((ACL(0,mode) & (ACL_READ | ACL_WRITE | ACL_EXECUTE)) << 1);
  acl_mode |= ((ACL(1,mode) & (ACL_READ | ACL_WRITE | ACL_EXECUTE)) >> 2);
  acl_mode |= ((ACL(2,mode) & (ACL_READ | ACL_WRITE | ACL_EXECUTE)) >> 5);

  inode->i_mode = ((inode->i_mode >> 9) << 9) | acl_mode;
  inode->i_dirt = 1;
  iput(inode);

  return 0;
}  
